﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using Aquiles.Core.Configuration;
using Aquiles.Cassandra07.Configuration;
using Aquiles.Core.Cluster;
using CassandraClient = Apache.Cassandra.Cassandra.Client;
using Apache.Cassandra;
using Aquiles.Cassandra07.Command;
using Aquiles.Helpers.Encoders;
using Aquiles.Cassandra07.Model;
using Aquiles.Core.Client;
using Aquiles.Helpers;
using Aquiles.Core.Exceptions;

namespace Aquiles.Cassandra07.Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class DataDrivenTest
    {
        private const string CLUSTERNAME = "TestCluster1";
        private const string KEYSPACENAME = "exampleKeyspace";
        private const string COLUMNFAMILYNAME = "exampleCF";
        public DataDrivenTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            KsDef keyspaceDefinition = new KsDef();
            keyspaceDefinition.Name = KEYSPACENAME;
            keyspaceDefinition.Replication_factor = 1;
            keyspaceDefinition.Strategy_class = "org.apache.cassandra.locator.SimpleStrategy";
            keyspaceDefinition.Cf_defs = new List<CfDef>();
            CfDef columnFamilyDefinition = new CfDef();
            columnFamilyDefinition.Keyspace = keyspaceDefinition.Name;
            columnFamilyDefinition.Comparator_type = "BytesType";
            columnFamilyDefinition.Key_cache_size = 20000;
            columnFamilyDefinition.Row_cache_save_period_in_seconds = 0;
            columnFamilyDefinition.Key_cache_save_period_in_seconds = 3600;
            columnFamilyDefinition.Memtable_flush_after_mins = 59;
            columnFamilyDefinition.Memtable_throughput_in_mb = 255;
            columnFamilyDefinition.Memtable_operations_in_millions = 0.29;
            columnFamilyDefinition.Name = COLUMNFAMILYNAME;

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            ExecutionBlock describe_keyspaces = new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.describe_keyspaces();
            });

            List<KsDef> keyspaceDefinitions = (List<KsDef>)cluster.Execute(describe_keyspaces);
            bool keyspaceExists = false;
            bool columnFamilyExists = false;
            if (keyspaceDefinitions != null)
            {
                foreach (KsDef ksDef in keyspaceDefinitions)
                {
                    if (ksDef.Name.CompareTo(keyspaceDefinition.Name) == 0)
                    {
                        keyspaceExists = true;
                        List<CfDef> columnFamilyDefinitions = ksDef.Cf_defs;
                        if (columnFamilyDefinitions != null)
                        {
                            foreach (CfDef cfDef in columnFamilyDefinitions)
                            {
                                if (cfDef.Name.CompareTo(columnFamilyDefinition.Name) == 0)
                                {
                                    columnFamilyExists = true;
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }
            }
            if (!keyspaceExists)
            {
                ExecutionBlock create_keyspace = new ExecutionBlock(delegate(CassandraClient client)
                {
                    return client.system_add_keyspace(keyspaceDefinition);
                });
                cluster.Execute(create_keyspace);
            }

            if (!columnFamilyExists)
            {
                ExecutionBlock create_columnFamily = new ExecutionBlock(delegate(CassandraClient client)
                {
                    return client.system_add_column_family(columnFamilyDefinition);
                });
                cluster.Execute(create_columnFamily, keyspaceDefinition.Name);
            }
        }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        public static void MyClassCleanup()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            ExecutionBlock drop_keyspace = new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.system_drop_keyspace(KEYSPACENAME);
            });
            cluster.Execute(drop_keyspace);
        }

        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        public void MyTestInitialize()
        {
            PopulateData();
        }

        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        private void PopulateData()
        {
            InsertCommand cmd = new InsertCommand();
            cmd.ColumnFamily = COLUMNFAMILYNAME;
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;

            for (long i = 0; i < 100; i++)
            {
                cmd.Key = ByteEncoderHelper.LongEncoder.ToByteArray(i);
                cmd.Column = new AquilesColumn();
                cmd.Column.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray("Data");
                cmd.Column.Value = cmd.Key;
                ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
                cluster.Execute(cmd, KEYSPACENAME);
            }
        }

        //[TestMethod]
        public void SimplestTestWithKeyRange()
        {
            GetKeyRangeSliceCommand cmd = new GetKeyRangeSliceCommand();
            cmd.ColumnFamily = COLUMNFAMILYNAME;
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            cmd.KeyTokenRange = new AquilesKeyRange();
            cmd.KeyTokenRange.Count = 10;
            cmd.KeyTokenRange.StartKey = ByteEncoderHelper.LongEncoder.ToByteArray(0);
            cmd.KeyTokenRange.EndKey = ByteEncoderHelper.LongEncoder.ToByteArray(50);
            cmd.Predicate = new AquilesSlicePredicate();
            cmd.Predicate.SliceRange = new AquilesSliceRange();
            cmd.Predicate.SliceRange.Count = int.MaxValue;
            cmd.Predicate.SliceRange.StartColumn = new byte[0];

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            cluster.Execute(cmd, KEYSPACENAME);

            Assert.IsNotNull(cmd.Output);
        }

        [TestMethod]
        public void TestWithKeyRangeGettingAllKeys()
        {
            GetKeyRangeSliceCommand cmd = new GetKeyRangeSliceCommand();
            cmd.ColumnFamily = COLUMNFAMILYNAME;
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            cmd.KeyTokenRange = new AquilesKeyRange();
            cmd.KeyTokenRange.Count = 10;
            cmd.KeyTokenRange.StartToken = "1";
            cmd.KeyTokenRange.EndToken = "1";
            cmd.Predicate = new AquilesSlicePredicate();
            cmd.Predicate.SliceRange = new AquilesSliceRange();
            cmd.Predicate.SliceRange.Count = int.MaxValue;
            cmd.Predicate.SliceRange.StartColumn = new byte[0];

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            cluster.Execute(cmd, KEYSPACENAME);

            Assert.IsNotNull(cmd.Output);
        }

        [TestMethod]
        public void SimpleTestInsertCheckDeleteCheck()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            byte[] key = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            byte[] columnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            byte[] columnValue = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            ColumnParent columnParent = new ColumnParent() {
                Column_family = COLUMNFAMILYNAME,
            };
            Column columnData = new Column() {
                Name = columnName,
                Timestamp = UnixHelper.UnixTimestamp,
                Value = columnValue,
            };

            // inserting data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                client.insert(key, columnParent, columnData, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            ColumnPath columnPath = new ColumnPath()
            {
                Column = columnName,
                Column_family = COLUMNFAMILYNAME,
            };
            ColumnOrSuperColumn columnOrSuperColumn = null;

            // getting inserted data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                columnOrSuperColumn = client.get(key, columnPath, ConsistencyLevel.ONE);
                return columnOrSuperColumn;
            }), KEYSPACENAME);

            Assert.IsNotNull(columnOrSuperColumn);
            Assert.IsNotNull(columnOrSuperColumn.Column);
            //Assert.AreEqual(columnOrSuperColumn.Column.Name, columnName);
            //Assert.AreEqual(columnOrSuperColumn.Column.Value, columnValue);

            // deleting data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                client.remove(key, columnPath, UnixHelper.UnixTimestamp, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            // verifying getting deleted data (there shouldn't be a response
            try
            {
                cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
                {
                    columnOrSuperColumn = client.get(key, columnPath, ConsistencyLevel.ONE);
                    return columnOrSuperColumn;
                }), KEYSPACENAME);
            }
            catch (ExecutionBlockException e)
            {
                Assert.AreEqual(e.InnerException.GetType(), typeof(NotFoundException));
                columnOrSuperColumn = null;
            }
            Assert.IsNull(columnOrSuperColumn);
        }
    }
}
