﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using Aquiles.Core.Configuration;
using Aquiles.Cassandra07.Configuration;
using Aquiles.Core.Cluster;
using Aquiles.Cassandra07.Model;
using Aquiles.Helpers.Encoders;
using Aquiles.Cassandra07.Command;
using Aquiles.Cassandra07.Command.System;
using Aquiles.Core.Client;

namespace Aquiles.Cassandra07.Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class IndexingTest
    {
        public const string CLUSTERNAME = "TestCluster1";
        public const string KEYSPACENAME = "Keyspace2";
        public const string COLUMNFAMILYNAME = "Indexed1";
        public const string INDEXED_COLUMNNUMBER = "IndexedColumn";
        public const string INDEXED_OTHERCOLUMNNAME = "OtherColumn";
        public const string COLUMNNAME_NAME = "Name";
        public const int BASEID = 1000;

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize()
        {
            AquilesColumnDefinition columnDefinition = null;
            Dictionary<string, AquilesColumnFamily> columnFamilies = new Dictionary<string, AquilesColumnFamily>();
            string comparator1 = "BytesType";
            AquilesColumnFamilyType columnFamilyType1 = AquilesColumnFamilyType.Standard;

            AquilesColumnFamily columnFamily = new AquilesColumnFamily();
            columnFamily.Columns = new List<AquilesColumnDefinition>();
            columnFamily.Name = COLUMNFAMILYNAME;
            columnFamily.Keyspace = KEYSPACENAME;
            columnFamily.Comparator = comparator1;
            columnFamily.Type = columnFamilyType1;

            columnDefinition = new AquilesColumnDefinition();
            columnDefinition.Name = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER);
            columnDefinition.IsIndex = true;
            columnDefinition.ValidationClass = "LongType";
            columnDefinition.IndexName = String.Concat(INDEXED_COLUMNNUMBER, "_idx");
            columnFamily.Columns.Add(columnDefinition);

            columnDefinition = new AquilesColumnDefinition();
            columnDefinition.Name = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_OTHERCOLUMNNAME);
            columnDefinition.IsIndex = true;
            columnDefinition.ValidationClass = "BytesType";
            columnDefinition.IndexName = String.Concat(INDEXED_OTHERCOLUMNNAME, "_idx");
            columnFamily.Columns.Add(columnDefinition);

            columnDefinition = new AquilesColumnDefinition();
            columnDefinition.ValidationClass = "UTF8Type";
            columnDefinition.Name = ByteEncoderHelper.UTF8Encoder.ToByteArray(COLUMNNAME_NAME);
            columnDefinition.IsIndex = false;
            columnFamily.Columns.Add(columnDefinition);
            columnFamilies.Add(columnFamily.Name, columnFamily);

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            RetrieveKeyspacesCommand retrieveKeyspacesCmd = new RetrieveKeyspacesCommand();
            cluster.Execute(retrieveKeyspacesCmd);
            bool keyspaceExists = false;
            for (int i = 0; (!keyspaceExists && i < retrieveKeyspacesCmd.Keyspaces.Count); i++)
            {
                keyspaceExists = (retrieveKeyspacesCmd.Keyspaces[i].Name.CompareTo(KEYSPACENAME) == 0);
            }
            if (keyspaceExists)
            {
                //Thread.Sleep(1000); // lets asleep for a while just in case cassandra needs some relaxing
                DropKeyspaceCommand dropKeyspaceCommand = new DropKeyspaceCommand();
                dropKeyspaceCommand.Keyspace = KEYSPACENAME;
                cluster.Execute(dropKeyspaceCommand);
                cluster.Execute(retrieveKeyspacesCmd);

                keyspaceExists = false;
                for (int i = 0; (!keyspaceExists && i < retrieveKeyspacesCmd.Keyspaces.Count); i++)
                {
                    keyspaceExists = (retrieveKeyspacesCmd.Keyspaces[i].Name.CompareTo(KEYSPACENAME) == 0);
                }
                Assert.IsFalse(keyspaceExists);
            }

            AddKeyspaceCommand addKeyspaceCmd = new AddKeyspaceCommand();
            addKeyspaceCmd.KeyspaceDefinition = new AquilesKeyspace()
            {
                Name = KEYSPACENAME,
                ReplicationFactor = 1,
                ColumnFamilies = columnFamilies
            };

            cluster.Execute(addKeyspaceCmd);
        }
        //
        // Use TestCleanup to run code after each test has run
        [TestCleanup()]
        public void MyTestCleanup()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            RetrieveKeyspacesCommand retrieveKeyspacesCmd = new RetrieveKeyspacesCommand();
            cluster.Execute(retrieveKeyspacesCmd);
            
            bool keyspaceExists = false;
            for (int i = 0; (!keyspaceExists && i < retrieveKeyspacesCmd.Keyspaces.Count); i++)
            {
                keyspaceExists = (retrieveKeyspacesCmd.Keyspaces[i].Name.CompareTo(KEYSPACENAME) == 0);
            }
            if (keyspaceExists)
            {
                DropKeyspaceCommand dropKeyspaceCommand = new DropKeyspaceCommand();
                dropKeyspaceCommand.Keyspace = KEYSPACENAME;
                cluster.Execute(dropKeyspaceCommand);
                cluster.Execute(retrieveKeyspacesCmd);

                keyspaceExists = false;
                for (int i = 0; (!keyspaceExists && i < retrieveKeyspacesCmd.Keyspaces.Count); i++)
                {
                    keyspaceExists = (retrieveKeyspacesCmd.Keyspaces[i].Name.CompareTo(KEYSPACENAME) == 0);
                }
                Assert.IsFalse(keyspaceExists);
            }
        }

        #endregion

        [TestMethod()]
        public void EqualsMethod()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            string name = null;
            BatchMutateCommand insertMutationCmd = new BatchMutateCommand();
            insertMutationCmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            insertMutationCmd.Mutations = new Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>>();

            AquilesColumn indexedColumn = new AquilesColumn();
            indexedColumn.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER);
            indexedColumn.TTL = 1500;

            AquilesColumn otherColumn = new AquilesColumn();
            otherColumn.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_OTHERCOLUMNNAME);
            otherColumn.TTL = 1500;

            AquilesColumn columnName = new AquilesColumn();
            columnName.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(COLUMNNAME_NAME);
            columnName.TTL = 1500;

            AquilesSetMutation setMutation = null;
            List<IAquilesMutation> mutations = null;
            Dictionary<string, List<IAquilesMutation>> cfMutations = null;
            for (long key = 0; key < 100; key++)
            {
                insertMutationCmd.Mutations.Clear();

                long divisiontrash = key % 2; // 1 for odd, 0 for even
                indexedColumn.Value = ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(divisiontrash);

                otherColumn.Value = ByteEncoderHelper.UTF8Encoder.ToByteArray("AT");

                name = String.Format("John{0000}", key);
                columnName.Value = ByteEncoderHelper.UTF8Encoder.ToByteArray(name);

                mutations = new List<IAquilesMutation>(2);

                setMutation = new AquilesSetMutation();
                setMutation.Column = indexedColumn;
                mutations.Add(setMutation);


                setMutation = new AquilesSetMutation();
                setMutation.Column = columnName;
                mutations.Add(setMutation);

                cfMutations = new Dictionary<string, List<IAquilesMutation>>();
                cfMutations.Add(COLUMNFAMILYNAME, mutations);

                insertMutationCmd.Mutations.Add(ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(key), cfMutations);

                cluster.Execute(insertMutationCmd, KEYSPACENAME);
            }

            GetIndexedSlicesCommand getIndexSliceCmd = new GetIndexedSlicesCommand();
            getIndexSliceCmd.ColumnFamily = COLUMNFAMILYNAME;
            getIndexSliceCmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            getIndexSliceCmd.IndexClause = new AquilesIndexClause();
            getIndexSliceCmd.IndexClause.Count = 100;
            getIndexSliceCmd.IndexClause.StartKey = new byte[0];
            getIndexSliceCmd.IndexClause.Expressions = new List<AquilesIndexExpression>();
            AquilesIndexExpression indexExpression = new AquilesIndexExpression();
            indexExpression.IndexOperator = AquilesIndexOperator.EQ;
            indexExpression.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER);
            indexExpression.Value = ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(0);
            getIndexSliceCmd.IndexClause.Expressions.Add(indexExpression);

            getIndexSliceCmd.Predicate = new AquilesSlicePredicate();
            getIndexSliceCmd.Predicate.Columns = new List<byte[]>(2);
            getIndexSliceCmd.Predicate.Columns.Add(ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER));
            getIndexSliceCmd.Predicate.Columns.Add(ByteEncoderHelper.UTF8Encoder.ToByteArray(COLUMNNAME_NAME));

            cluster.Execute(getIndexSliceCmd, KEYSPACENAME);

            Assert.IsNotNull(getIndexSliceCmd.Output);
            Assert.AreEqual(50, getIndexSliceCmd.Output.Count);
            foreach (GetIndexedSlicesCommand.Out out1 in getIndexSliceCmd.Output)
            {
                string blah = String.Format("key={0}, {1}={2},{3}={4}",
                    ByteEncoderHelper.BigEndianLongEncoder.FromByteArray(out1.Key),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[0].Column.ColumnName),
                    ByteEncoderHelper.BigEndianLongEncoder.FromByteArray(out1.Columns[0].Column.Value),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[1].Column.ColumnName),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[1].Column.Value));

                Console.WriteLine(blah);
            }
        }

        [TestMethod()]
        public void GreaterThanTest()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            string name = null;
            BatchMutateCommand insertMutationCmd = new BatchMutateCommand();
            insertMutationCmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            insertMutationCmd.Mutations = new Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>>();

            AquilesColumn indexedColumn = new AquilesColumn();
            indexedColumn.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER);
            indexedColumn.TTL = 1500;

            AquilesColumn otherColumn = new AquilesColumn();
            otherColumn.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_OTHERCOLUMNNAME);
            otherColumn.TTL = 1500;

            AquilesColumn columnName = new AquilesColumn();
            columnName.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(COLUMNNAME_NAME);
            columnName.TTL = 1500;

            AquilesSetMutation setMutation = null;
            List<IAquilesMutation> mutations = null;
            Dictionary<string, List<IAquilesMutation>> cfMutations = null;
            for (long key = 0; key < 100; key++)
            {
                insertMutationCmd.Mutations.Clear();

                long divisiontrash = key % 5;
                indexedColumn.Value = ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(divisiontrash);
                name = String.Format("John{0000}", key);
                columnName.Value = ByteEncoderHelper.UTF8Encoder.ToByteArray(name);
                otherColumn.Value = ByteEncoderHelper.UTF8Encoder.ToByteArray("AT");

                mutations = new List<IAquilesMutation>(3);

                setMutation = new AquilesSetMutation();
                setMutation.Column = indexedColumn;
                mutations.Add(setMutation);

                setMutation = new AquilesSetMutation();
                setMutation.Column = columnName;
                mutations.Add(setMutation);

                setMutation = new AquilesSetMutation();
                setMutation.Column = otherColumn;
                mutations.Add(setMutation);

                cfMutations = new Dictionary<string, List<IAquilesMutation>>();
                cfMutations.Add(COLUMNFAMILYNAME, mutations);

                insertMutationCmd.Mutations.Add(ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(key), cfMutations);

                cluster.Execute(insertMutationCmd, KEYSPACENAME);
            }

            GetIndexedSlicesCommand getIndexSliceCmd = new GetIndexedSlicesCommand();
            getIndexSliceCmd.ColumnFamily = COLUMNFAMILYNAME;
            getIndexSliceCmd.ConsistencyLevel = AquilesConsistencyLevel.ONE;
            getIndexSliceCmd.IndexClause = new AquilesIndexClause();
            getIndexSliceCmd.IndexClause.Count = 100;
            getIndexSliceCmd.IndexClause.StartKey = new byte[0];
            getIndexSliceCmd.IndexClause.Expressions = new List<AquilesIndexExpression>();

            AquilesIndexExpression indexExpression = new AquilesIndexExpression();
            indexExpression.IndexOperator = AquilesIndexOperator.EQ;
            indexExpression.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_OTHERCOLUMNNAME);
            indexExpression.Value = ByteEncoderHelper.UTF8Encoder.ToByteArray("AT");
            getIndexSliceCmd.IndexClause.Expressions.Add(indexExpression);

            indexExpression = new AquilesIndexExpression();
            indexExpression.IndexOperator = AquilesIndexOperator.GTE;
            indexExpression.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER);
            indexExpression.Value = ByteEncoderHelper.BigEndianLongEncoder.ToByteArray(4);
            getIndexSliceCmd.IndexClause.Expressions.Add(indexExpression);

            getIndexSliceCmd.Predicate = new AquilesSlicePredicate();
            getIndexSliceCmd.Predicate.Columns = new List<byte[]>(2);
            getIndexSliceCmd.Predicate.Columns.Add(ByteEncoderHelper.UTF8Encoder.ToByteArray(INDEXED_COLUMNNUMBER));
            getIndexSliceCmd.Predicate.Columns.Add(ByteEncoderHelper.UTF8Encoder.ToByteArray(COLUMNNAME_NAME));

            cluster.Execute(getIndexSliceCmd, KEYSPACENAME);

            Assert.IsNotNull(getIndexSliceCmd.Output);
            Assert.AreEqual(20, getIndexSliceCmd.Output.Count);
            foreach (GetIndexedSlicesCommand.Out out1 in getIndexSliceCmd.Output)
            {
                string blah = String.Format("key={0}, {1}={2}, {3}={4}",
                    ByteEncoderHelper.BigEndianLongEncoder.FromByteArray(out1.Key),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[0].Column.ColumnName),
                    ByteEncoderHelper.BigEndianLongEncoder.FromByteArray(out1.Columns[0].Column.Value),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[1].Column.ColumnName),
                    ByteEncoderHelper.UTF8Encoder.FromByteArray(out1.Columns[1].Column.Value));

                Console.WriteLine(blah);
            }
        }
    }
}
