﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Aquiles.Core.Client;
using CassandraClient = Apache.Cassandra.Cassandra.Client;
using Aquiles.Core.Cluster;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;

namespace Aquiles.Cassandra07.Test
{
    /// <summary>
    /// Summary description for CommandsTest
    /// </summary>
    [TestClass]
    public class SimpleCommandsTest
    {
        private const string CLUSTERNAME = "TestCluster1";

        public SimpleCommandsTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            AquilesHelper.Initialize();
        }

        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void AccessingNonExistantKeyspace()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            try
            {
                cluster.Execute(new ExecutionBlock(this.getClusterName), "lala");
            }
            catch (ExecutionBlockException ex)
            {
                // this is expected
            }
        }

        [TestMethod]
        public void AccessingNonExistantKeyspaceBorrowingClient()
        {
            bool isClientHealthy = true, noException = true;
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            IClient client = cluster.Borrow("lala");
            try
            {
                client.Execute(new ExecutionBlock(this.getClusterName));
            }
            catch (ExecutionBlockException ex)
            {
                noException = false;
                isClientHealthy = ex.IsClientHealthy;
            }
            finally
            {
                if (noException || isClientHealthy)
                {
                    cluster.Release(client);
                }
                else
                {
                    cluster.Invalidate(client);
                }
            }
        }

        [TestMethod]
        public void SimpleGetClusterNameExecutionBlockOnClient()
        {
            string clusterName2 = null;
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            clusterName2 = (string) cluster.Execute(new ExecutionBlock(this.getClusterName));
            Assert.IsNotNull(clusterName2);
        }

        [TestMethod]
        public void SimpleGetClusterNameExecutionBlockOnCluster()
        {

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            string clusterName2 = (string) cluster.Execute(new ExecutionBlock(this.getClusterName));
            Assert.IsNotNull(clusterName2);
        }

        [TestMethod]
        public void ABMOnKeyspaceAndColumnFamily()
        {
            KsDef keyspaceDefinition = new KsDef();
            keyspaceDefinition.Name = "exampleKeyspace";
            keyspaceDefinition.Replication_factor = 1;
            keyspaceDefinition.Strategy_class = "org.apache.cassandra.locator.SimpleStrategy";
            keyspaceDefinition.Cf_defs = new List<CfDef>();
            CfDef columnFamilyDefinition = new CfDef();
            columnFamilyDefinition.Keyspace = keyspaceDefinition.Name;
            columnFamilyDefinition.Comparator_type = "BytesType";
            columnFamilyDefinition.Key_cache_size = 20000;
            columnFamilyDefinition.Row_cache_save_period_in_seconds = 0;
            columnFamilyDefinition.Key_cache_save_period_in_seconds = 3600;
            columnFamilyDefinition.Memtable_flush_after_mins = 59;
            columnFamilyDefinition.Memtable_throughput_in_mb = 255;
            columnFamilyDefinition.Memtable_operations_in_millions = 0.29;
            columnFamilyDefinition.Name = "exampleCF";

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            ExecutionBlock describe_keyspaces = new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.describe_keyspaces();
            });

            List<KsDef> keyspaceDefinitions = (List<KsDef>) cluster.Execute(describe_keyspaces);
            bool keyspaceExists = false;
            bool columnFamilyExists = false;
            if (keyspaceDefinitions != null) {
                foreach(KsDef ksDef in keyspaceDefinitions) {
                    if (ksDef.Name.CompareTo(keyspaceDefinition.Name) == 0)
                    {
                        keyspaceExists = true;
                        List<CfDef> columnFamilyDefinitions = ksDef.Cf_defs;
                        if (columnFamilyDefinitions != null) {
                            foreach(CfDef cfDef in columnFamilyDefinitions) {
                                if (cfDef.Name.CompareTo(columnFamilyDefinition.Name) == 0) {
                                    columnFamilyExists = true;
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }
            }
            try
            {
                if (!keyspaceExists)
                {
                    ExecutionBlock create_keyspace = new ExecutionBlock(delegate(CassandraClient client)
                    {
                        return client.system_add_keyspace(keyspaceDefinition);
                    });
                    cluster.Execute(create_keyspace);
                }

                if (!columnFamilyExists)
                {
                    ExecutionBlock create_columnFamily = new ExecutionBlock(delegate(CassandraClient client)
                    {
                        return client.system_add_column_family(columnFamilyDefinition);
                    });
                    cluster.Execute(create_columnFamily, keyspaceDefinition.Name);
                }
            }
            finally
            {
                ExecutionBlock drop_keyspace = new ExecutionBlock(delegate(CassandraClient client)
                    {
                        return client.system_drop_keyspace(keyspaceDefinition.Name);
                    });
                cluster.Execute(drop_keyspace);
            }
        }

        public string getClusterName(CassandraClient client)
        {
            return client.describe_cluster_name();
        }
    }
}
