﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Aquiles.Cassandra07.Model;
using Aquiles.Cassandra07.Command;
using System.Threading;
using Aquiles.Cassandra07.Command.System;
using IAquilesConnection = Aquiles.Core.Client.IClient;
using System.Configuration;
using Aquiles.Helpers.Encoders;
using Aquiles.Core.Cluster;

namespace Aquiles.Cassandra07.Win32.Test
{
    class Program
    {
        private static int numConcurrentThreads;
        private static volatile int numberCycles;

        private static bool shouldCreateKeyspace;
        private static string keySpace;
        private static string columnFamily;
        private static string columnName;
        private static string columnValue;

        private static bool keyspaceCreated = false;
        private static bool columnFamilyCreated = false;

        static void Main(string[] args)
        {
            //for (int i = 0; i < 100; i++)
            //{
            //    Guid guid = Aquiles.Helpers.GuidGenerator.GenerateTimeBasedGuid();
            //    Console.WriteLine(guid.ToString());
            //}
            process(args);
        }

        private static void process(string[] args)
        {
            bool okToProceed = false;
            ShowTitle();
            try
            {
                // lets initialize Aquiles
                AquilesHelper.Initialize();
                ParseInput(args);
                ParseConfiguration();
                okToProceed = true;
            }
            catch (Exception ex)
            {
                ShowInputHelp(ex);
            }


            if (okToProceed)
            {
                if (shouldCreateKeyspace)
                {
                    CreateTestStructure();
                }
                ShowInputs();
                Console.Write("Press any key to start...");
                Console.Read();

                ManualResetEvent[] signals = new ManualResetEvent[numConcurrentThreads];

                for (int i = 0; i < numConcurrentThreads; i++)
                {
                    signals[i] = new ManualResetEvent(false);
                    ThreadPool.QueueUserWorkItem(new WaitCallback(TesterThreadRun), signals[i]);
                }

                if (signals.Length <= 64)
                {
                    WaitHandle.WaitAll(signals);
                }
                else
                {
                    // HACK!! =)
                    int from = 0;
                    int remainning = signals.Length;
                    ManualResetEvent[] waitHandleSignals;
                    do
                    {
                        waitHandleSignals = new ManualResetEvent[Math.Min(remainning, 64)];
                        Array.Copy(signals, from, waitHandleSignals, 0, waitHandleSignals.Length);
                        WaitHandle.WaitAll(waitHandleSignals);
                        from += 64;
                        remainning = signals.Length - from;
                    } while (remainning > 0);
                }


                if (shouldCreateKeyspace)
                {
                    DeleteTestStructure();
                }
            }
            Console.ReadKey();
        }

        private static void DeleteTestStructure()
        {
            if (keyspaceCreated)
            {
                RemoveKeyspace(keySpace);
            }
            else if (columnFamilyCreated)
            {
                RemoveColumnFamily(keySpace, columnFamily);
            }

        }

        private static void RemoveColumnFamily(string keySpace, string columnFamily)
        {
            DropColumnFamilyCommand cmd = new DropColumnFamilyCommand();
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ALL;
            cmd.ColumnFamily = columnFamily;
            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");
            cluster.Execute(cmd, keySpace);
            Console.WriteLine("Remove ColumnFamily result: " + cmd.Output);
        }

        private static void RemoveKeyspace(string keySpace)
        {
            DropKeyspaceCommand cmd = new DropKeyspaceCommand();
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ALL;
            cmd.Keyspace = keySpace;
            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");
            cluster.Execute(cmd, keySpace);
            Console.WriteLine("Remove Keypsace result: " + cmd.Output);
        }

        private static void CreateTestStructure()
        {
            Console.WriteLine("Creating Keyspace {0} with ColumnFamily {1}", keySpace, columnFamily);
            AquilesKeyspace keyspace = retrieveIfExists(keySpace);
            if (keyspace == null)
            {
                createKeyspace(keySpace);
                keyspaceCreated = true;

                createColumnFamily(columnFamily);
                columnFamilyCreated = true;
            }
            else if (!keyspace.ColumnFamilies.ContainsKey(columnFamily))
            {
                createColumnFamily(columnFamily);
                columnFamilyCreated = true;
            }
        }

        private static void createColumnFamily(string columnFamily)
        {
            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");

            AddColumnFamilyCommand cmd = new AddColumnFamilyCommand();
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ALL;
            cmd.ColumnFamilyDefinition = new AquilesColumnFamily();
            cmd.ColumnFamilyDefinition.Name = columnFamily;
            cmd.ColumnFamilyDefinition.Keyspace = keySpace;
            cmd.ColumnFamilyDefinition.Comparator = "BytesType";
            cmd.ColumnFamilyDefinition.Type = AquilesColumnFamilyType.Standard;
            cmd.ColumnFamilyDefinition.RowCacheSize = 20;

            cluster.Execute(cmd, keySpace);

            Console.WriteLine("Add ColumnFamily result: " + cmd.Output);
        }

        private static void createKeyspace(string keySpace)
        {
            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");

            AddKeyspaceCommand cmd = new AddKeyspaceCommand();
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ALL;
            cmd.KeyspaceDefinition = new AquilesKeyspace();
            cmd.KeyspaceDefinition.Name = keySpace;
            cmd.KeyspaceDefinition.ReplicationFactor = 1;
            cluster.Execute(cmd); 
            Console.WriteLine("Add Keyspace result: " + cmd.Output);
        }

        private static AquilesKeyspace retrieveIfExists(string keySpace)
        {
            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");
            RetrieveKeyspacesCommand cmd = new RetrieveKeyspacesCommand();
            cmd.ConsistencyLevel = AquilesConsistencyLevel.ANY;
            cluster.Execute(cmd);

            if (cmd.Keyspaces != null)
            {
                foreach (AquilesKeyspace keyspace in cmd.Keyspaces)
                {
                    if (String.Compare(keyspace.Name, keySpace) == 0)
                    {
                        return keyspace;
                    }
                }
            }
            return null;
        }

        private static void ParseConfiguration()
        {
            string tempValue = ConfigurationManager.AppSettings["createKeyspace"];
            if (String.IsNullOrEmpty(tempValue) || !Boolean.TryParse(tempValue, out shouldCreateKeyspace))
            {
                shouldCreateKeyspace = false;
            }

            keySpace = ConfigurationManager.AppSettings["Keyspace"];
            if (String.IsNullOrEmpty(keySpace))
            {
                throw new ConfigurationErrorsException("Keyspace must not be null nor empty.");
            }
            columnFamily = ConfigurationManager.AppSettings["ColumnFamily"];
            if (String.IsNullOrEmpty(columnFamily))
            {
                throw new ConfigurationErrorsException("ColumnFamily must not be null nor empty.");
            }
            columnName = ConfigurationManager.AppSettings["ColumnName"];
            if (String.IsNullOrEmpty(columnName))
            {
                throw new ConfigurationErrorsException("ColumnName must not be null nor empty.");
            }
            int dataSize = 0;
            if (!int.TryParse(ConfigurationManager.AppSettings["DataSize"], out dataSize))
            {
                throw new ConfigurationErrorsException("Datasize in AppSettings must be a valid integer greater than 0.");
            }
            columnValue = new String('A', dataSize);
        }

        private static void ShowInputs()
        {
            int workerThreads = 0;
            int completitionPortThreads = 0;
            ThreadPool.GetAvailableThreads(out workerThreads, out completitionPortThreads);
            //ThreadPool.GetMaxThreads(out maxThreads, out completitionPortThreads);
            Console.WriteLine("Concurrent Threads: {0:0000} - Cycles: {0:0000000000} - ThreadPool available threads: {2:0000000000}", numConcurrentThreads, numConcurrentThreads, workerThreads);
        }

        private static void ShowInputHelp(Exception ex)
        {
            Console.WriteLine("Exception: {0}\n", ex);
            Console.WriteLine("\tAquiles.Win32.Test.exe <CONCURRENTTHREADS> <CYCLES>");
            Console.WriteLine("\tExample:");
            Console.WriteLine("\t\tAquiles.Win32.Test.exe 10 1000\n");
        }

        private static void ShowTitle()
        {
            Console.WriteLine("Aquiles.Win32.Test");
            Console.WriteLine("==================\n");
        }

        private static void ParseInput(string[] args)
        {
            int input = 0;
            if (args.Length == 2)
            {
                if (!int.TryParse(args[0], out input))
                {
                    throw new ArgumentException("ConcurrentThread must be a valid int number.", "CONCURRENTTHREADS");
                }
                else if (input <= 0)
                {
                    throw new ArgumentException("ConcurrentThread must be greater than 0.", "CONCURRENTTHREADS");
                }
                numConcurrentThreads = input;

                if (!int.TryParse(args[1], out input))
                {
                    throw new ArgumentException("Cycles must be a valid int number.", "CYCLES");
                }
                else if (input <= 0)
                {
                    throw new ArgumentException("Cycles must be greater than 0.", "CYCLES");
                }

                numberCycles = input;
            }
            else
            {
                throw new ArgumentException("No arguments specified or argments quantity does not match.");
            }
        }

        private static void ShowPendingCycles()
        {
            Console.WriteLine("Thread['{0}']: Pending cycles: {1:0000000000}", Thread.CurrentThread.ManagedThreadId, numberCycles);
        }

        private static void TesterThreadRun(object state)
        {
            ManualResetEvent mySignal = (ManualResetEvent)state;
            while (numberCycles > 0)
            {
                ShowPendingCycles();
                try
                {
                    TestMultipleConnections(keySpace, columnFamily, Guid.NewGuid().ToString(), columnName, columnValue);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Thread['{0}']: {1}", Thread.CurrentThread.ManagedThreadId, ex);
                }
                finally
                {
                    numberCycles--;
                }
            }
            mySignal.Set();
        }



        private static void TestMultipleConnections(string keyspace, string columnFamily, string key, string columnName, string columnValue)
        {
            InsertCommand insertCommand;
            GetCommand getCommand;
            DeleteCommand delCommand;

            PrepareCommands(columnFamily, key, columnName, columnValue, out insertCommand, out getCommand, out delCommand);

            ICluster cluster = AquilesHelper.RetrieveCluster("TestCluster1");

            cluster.Execute(insertCommand, keySpace);
            cluster.Execute(getCommand, keySpace);

            GetCommand.Out output = getCommand.Output;


            if ((output != null)
                && (output.Column != null)
                && (columnName.CompareTo(ByteEncoderHelper.UTF8Encoder.FromByteArray(output.Column.ColumnName)) == 0)
                && (columnValue.CompareTo(ByteEncoderHelper.UTF8Encoder.FromByteArray(output.Column.Value)) == 0))
            {
                delCommand.Column = output.Column;

                cluster.Execute(delCommand, keySpace);

                cluster.Execute(getCommand, keySpace);

                if (getCommand.Output != null)
                {
                    throw new Exception(String.Format("Delete command failed with key '{0}'", key));
                }
            }
            else
            {
                throw new Exception(String.Format("Couldn't get the columndata from Cassandra with key '{0}'", key));
            }
        }

        private static void PrepareCommands(string columnFamily, string key, string columnName, string columnValue, out InsertCommand insertCommand, out GetCommand getCommand, out DeleteCommand delCommand)
        {
            // Insert statement
            insertCommand = new InsertCommand();
            insertCommand.ColumnFamily = columnFamily;
            insertCommand.Key = ByteEncoderHelper.UTF8Encoder.ToByteArray(key);
            insertCommand.Column = new Aquiles.Cassandra07.Model.AquilesColumn()
            {
                ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(columnName),
                Value = ByteEncoderHelper.UTF8Encoder.ToByteArray(columnValue),
                TTL = 60
            };

            // Get statement

            getCommand = new GetCommand();
            getCommand.Key = ByteEncoderHelper.UTF8Encoder.ToByteArray(key);
            getCommand.ColumnFamily = columnFamily;
            getCommand.ColumnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(columnName);

            //Delete statement
            delCommand = new DeleteCommand();
            delCommand.Key = ByteEncoderHelper.UTF8Encoder.ToByteArray(key);
            delCommand.ColumnFamily = columnFamily;
        }
    }
}
