﻿using System;
using System.Collections.Generic;

using System.Text;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;
using Aquiles.Core.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Abstract class for an AquilesCommand that needs to have a Keyspace, a ColumnFamily and a Key
    /// </summary>
    public abstract class AbstractKeyspaceColumnFamilyKeyDependantCommand : AbstractKeyspaceColumnFamilyDependantCommand
    {
        /// <summary>
        /// get or set the Key
        /// </summary>
        public byte[] Key
        {
            set;
            get;
        }

        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public new virtual void ValidateInput()
        {
            base.ValidateInput();
            if (this.Key == null || this.Key.Length == 0)
            {
                throw new AquilesException("Key must be not null or empty.");
            }
        }
    }
}
