﻿using System;
using System.Collections.Generic;

using System.Text;
using Apache.Cassandra;
using Aquiles.Core.Model;
using Aquiles.Core.Exceptions;
using Aquiles.Helpers;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to delete a Column, SuperColumn or a Key from Keyspace of a given cluster
    /// </summary>
    public class DeleteCommand : AbstractKeyspaceColumnFamilyKeyDependantCommand, IAquilesCommand
    {

        /// <summary>
        /// get or set supercolumn name
        /// </summary>
        public byte[] SuperColumnName
        {
            set;
            get;
        }
        /// <summary>
        /// get or set Column information
        /// </summary>
        public AquilesColumn Column
        {
            set;
            get;
        }

        /// <summary>
        /// ctor
        /// </summary>
        public DeleteCommand() : base()
        {
        }

        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "remove" over the connection. No return values
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            ColumnPath columnPath;
            if (this.Column != null)
            {
                columnPath = this.BuildColumnPath(this.ColumnFamily, this.SuperColumnName, this.Column.ColumnName);
                cassandraClient.remove(this.Key, columnPath, this.Column.Timestamp.Value, this.GetCassandraConsistencyLevel());
            }
            else
            {
                columnPath = this.BuildColumnPath(this.ColumnFamily, this.SuperColumnName, null);
                cassandraClient.remove(this.Key, columnPath, UnixHelper.UnixTimestamp, this.GetCassandraConsistencyLevel());
            }
        }

        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public override void ValidateInput()
        {
            base.ValidateInput();
            if (this.Column != null)
            {
                this.Column.ValidateForDeletationOperation();
                if (!this.Column.Timestamp.HasValue)
                {
                    throw new AquilesException("If input Column exists, must have a valid Timestamp.");
                }
            }
        }
        #endregion
    }
}
