﻿using System;
using System.Collections.Generic;

using System.Text;
using Apache.Cassandra;
using Aquiles.Core.Model;
using Aquiles.Core.Exceptions;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to retrieve a Column or SuperColumn from Keyspace of a given cluster with the given key
    /// </summary>
    public class GetCommand : AbstractKeyspaceColumnFamilyKeyDependantCommand, IAquilesCommand
    {
        /// <summary>
        /// get the return value
        /// </summary>
        public Out Output
        {
            get;
            private set;
        }

        /// <summary>
        /// get or set the SuperColumn
        /// </summary>
        public byte[] SuperColumnName
        {
            set;
            get;
        }

        /// <summary>
        /// get or set the ColumnName
        /// </summary>
        public byte[] ColumnName
        {
            set;
            get;
        }

        /// <summary>
        /// ctor
        /// </summary>
        public GetCommand() : base()
        {
        }


        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "get" over the connection. Return values are set into Output
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            ColumnOrSuperColumn columnOrSupercolumn = null;
            ColumnPath columnPath = this.BuildColumnPath(this.ColumnFamily, this.SuperColumnName, this.ColumnName);
            try
            {
                columnOrSupercolumn = cassandraClient.get(this.Key, columnPath, this.GetCassandraConsistencyLevel());
            }
            catch (NotFoundException) { }

            if (columnOrSupercolumn != null)
            {
                this.buildOut(columnOrSupercolumn);
            }
            else
            {
                // in case of reuse of the command
                this.Output = null;
            }
        }

        #endregion

        private void buildOut(ColumnOrSuperColumn columnOrSupercolumn)
        {
            Out outResponse = new Out();
            outResponse.Column = ModelConverterHelper.Convert<AquilesColumn, Column>(columnOrSupercolumn.Column);
            outResponse.SuperColumn = ModelConverterHelper.Convert<AquilesSuperColumn, SuperColumn>(columnOrSupercolumn.Super_column);
            this.Output = outResponse;
        }

        /// <summary>
        /// structure to Return Values
        /// </summary>
        public class Out {
            /// <summary>
            /// get or set Column Information
            /// </summary>
            public AquilesColumn Column
            {
                get;
                set;
            }
            /// <summary>
            /// get or set SuperColumn Information
            /// </summary>
            public AquilesSuperColumn SuperColumn
            {
                get;
                set;
            }
        }



    }
}
