﻿using System;
using System.Collections.Generic;

using System.Text;
using Apache.Cassandra;
using System.IO;
using Aquiles.Core.Model;
using Aquiles.Core.Exceptions;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to insert a Column into a Keyspace of a given cluster
    /// </summary>
    public class InsertCommand : AbstractKeyspaceColumnFamilyKeyDependantCommand, IAquilesCommand
    {
        /// <summary>
        /// get or set the name of the supercolumn
        /// </summary>
        public byte[] SuperColumn
        {
            set;
            get;
        }

        /// <summary>
        /// get or set the column information
        /// </summary>
        public AquilesColumn Column
        {
            set;
            get;
        }

        /// <summary>
        /// ctor
        /// </summary>
        public InsertCommand() : base()
        {
        }

        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "insert" over the connection. No return values.
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            Column column = ModelConverterHelper.Convert<AquilesColumn,Column>(this.Column);
            ColumnParent columnParent = this.BuildColumnParent(this.SuperColumn);
            cassandraClient.insert(this.Key, columnParent, column, this.GetCassandraConsistencyLevel());

        }
        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public override void ValidateInput()
        {
            base.ValidateInput();
            if (this.Column == null)
            {
                throw new AquilesException("Column parameter must have a value.");
            }
            this.Column.ValidateForInsertOperation();
        }
        #endregion

    }
}
