﻿using System;
using System.Collections.Generic;

using System.Text;
using Aquiles.Core.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to retrieve the ClusterName from a Cluster (the real name)
    /// </summary>
    public class RetrieveClusterNameCommand : AbstractCommand, IAquilesCommand
    {
        /// <summary>
        /// Get the ClusterName
        /// </summary>
        public string ClusterName
        {
            get;
            private set;
        }

        #region IAquilesCommand Members

        /// <summary>
        /// Executes a "describe_cluster_name" over the connection, set the ClusterName property with the returned value.
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Apache.Cassandra.Cassandra.Client cassandraClient)
        {
            this.ClusterName = cassandraClient.describe_cluster_name();
        }

        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public void ValidateInput()
        {
            //do nothing
        }

        #endregion
    }
}
