﻿using System;
using System.Collections.Generic;

using System.Text;
using Thrift.Collections;
using Apache.Cassandra;
using Aquiles.Core.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to retrieve the partitioner for a cluster
    /// </summary>
    public class RetrieveClusterPartitionerCommand : AbstractCommand, IAquilesCommand
    {
        /// <summary>
        /// get the configured Partitioner
        /// </summary>
        public string Partitioner
        {
            get;
            private set;
        }

        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "describe_partitioner" over the connection, set the Partitioner property with the returned value.
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            string partitioner = cassandraClient.describe_partitioner();
            this.Partitioner = partitioner;
        }
        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public void ValidateInput()
        {
            //do nothing
        }

        #endregion
    }
}
