﻿using System;
using System.Collections.Generic;

using System.Text;
using Thrift.Collections;
using Apache.Cassandra;
using Aquiles.Core.Model;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to retrieve the list of keyspaces for a cluster
    /// </summary>
    public class RetrieveKeyspacesCommand : AbstractCommand, IAquilesCommand
    {
        /// <summary>
        /// get the list of keyspaces
        /// </summary>
        public List<AquilesKeyspace> Keyspaces
        {
            get;
            private set;
        }

        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "describe_keyspaces" over the connection, set the Keyspaces property with the returned value.
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            List<KsDef> keySpaces = cassandraClient.describe_keyspaces();
            this.Keyspaces = this.buildKeyspaces(keySpaces);
        }

        private List<AquilesKeyspace> buildKeyspaces(List<KsDef> keySpaces)
        {
            List<AquilesKeyspace> convertedKeyspaces = null;
            if (keySpaces != null)
            {
                convertedKeyspaces = new List<AquilesKeyspace>(keySpaces.Count);
                foreach (KsDef keyspace in keySpaces)
                {
                    convertedKeyspaces.Add(ModelConverterHelper.Convert<AquilesKeyspace, KsDef>(keyspace));
                }
            }
            else
            {
                convertedKeyspaces = null;
            }
            return convertedKeyspaces;
        }
        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public void ValidateInput()
        {
            //do nothing
        }

        #endregion
    }
}
