﻿using System;
using System.Collections.Generic;

using System.Text;
using Aquiles.Core.Model;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command.System
{
    /// <summary>
    /// Command to update a Keyspace structure from a Cluster
    /// </summary>
    public class UpdateKeyspaceCommand : AbstractCommand, IAquilesCommand
    {
        /// <summary>
        /// Get or Set the KeyspaceDefinition
        /// </summary>
        public AquilesKeyspace KeyspaceDefinition
        {
            get;
            set;
        }

        /// <summary>
        /// get the command output
        /// </summary>
        public string Output
        {
            get;
            private set;
        }

        #region IAquilesSystemCommand Members

        /// <summary>
        /// Executes a "system_update_keyspace" over the connection, set the ClusterName property with the returned value.
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Apache.Cassandra.Cassandra.Client cassandraClient)
        {
            KsDef keyspace = ModelConverterHelper.Convert<AquilesKeyspace, KsDef>(this.KeyspaceDefinition);
            this.Output = cassandraClient.system_update_keyspace(keyspace);
        }

        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public void ValidateInput()
        {
            this.KeyspaceDefinition.ValidateForInsertOperation();
        }

        #endregion
    }
}
