﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Aquiles.Cassandra07.Model;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;

namespace Aquiles.Cassandra07.Converter.Model.Impl
{
    /// <summary>
    /// Converter for AquilesMutation
    /// </summary>
    public class AquilesMutationConverter : IThriftConverter<IAquilesMutation, Mutation>
    {

        #region IThriftConverter<IAquilesMutation,Mutation> Members

        /// <summary>
        /// Transform IAquilesMutation structure into Mutation
        /// </summary>
        /// <param name="objectA"></param>
        /// <returns></returns>
        public Mutation Transform(IAquilesMutation objectA)
        {
            Mutation mutation = null;
            Type objectAType = objectA.GetType();
            if (objectAType.Equals(typeof(AquilesSetMutation)))
            {
                mutation = this.convertSetMutation((AquilesSetMutation)objectA);
            }
            else if (objectAType.Equals(typeof(AquilesDeletionMutation)))
            {
                mutation = this.convertDeletionMutation((AquilesDeletionMutation)objectA);
            }
            else
            {
                throw new AquilesException("Mutation converter not implemented.");
            }
            return mutation;
        }

        /// <summary>
        /// Transform Mutation structure into IAquilesMutation
        /// Not implemented! Why would you need this??? 
        /// </summary>
        /// <param name="objectB"></param>
        /// <returns></returns>
        public IAquilesMutation Transform(Mutation objectB)
        {
            throw new NotImplementedException();
        }

        #endregion

        private Mutation convertSetMutation(AquilesSetMutation mutation)
        {
            ColumnOrSuperColumn columnOrSuperColumn = new ColumnOrSuperColumn();
            if (mutation.Column != null)
            {
                columnOrSuperColumn.Column = ModelConverterHelper.Convert<AquilesColumn,Column>(mutation.Column);
            }

            if (mutation.SuperColumn != null)
            {
                columnOrSuperColumn.Super_column = ModelConverterHelper.Convert<AquilesSuperColumn,SuperColumn>(mutation.SuperColumn);
            }

            return new Mutation()
                {
                    Column_or_supercolumn = columnOrSuperColumn
                };
        }

        private Mutation convertDeletionMutation(AquilesDeletionMutation aquilesDeletionMutation)
        {
            Deletion deletion = new Deletion();
            deletion.Predicate = ModelConverterHelper.Convert<AquilesSlicePredicate,SlicePredicate>(aquilesDeletionMutation.Predicate);
            deletion.Super_column = aquilesDeletionMutation.SuperColumn;
            if (aquilesDeletionMutation.Timestamp.HasValue)
            {
                deletion.Timestamp = aquilesDeletionMutation.Timestamp.Value;
            }

            return new Mutation() { Deletion = deletion };
        }
    }
}
