﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using Aquiles.Core.Configuration;
using Aquiles.Core.Cluster;
using CassandraClient = Apache.Cassandra.Cassandra.Client;
using Apache.Cassandra;
using Aquiles.Helpers.Encoders;
using Aquiles.Core.Client;
using Aquiles.Helpers;
using Aquiles.Core.Exceptions;

namespace Aquiles.Cassandra08.Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class DataDrivenTest
    {
        private const string CLUSTERNAME = "TestCluster1";
        private const string KEYSPACENAME = "exampleKeyspace";
        private const string COLUMNFAMILYNAME = "exampleCF";
        public DataDrivenTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            KsDef keyspaceDefinition = new KsDef();
            keyspaceDefinition.Name = KEYSPACENAME;
            keyspaceDefinition.Replication_factor = 1;
            keyspaceDefinition.Strategy_class = "org.apache.cassandra.locator.SimpleStrategy";
            keyspaceDefinition.Cf_defs = new List<CfDef>();
            CfDef columnFamilyDefinition = new CfDef();
            columnFamilyDefinition.Keyspace = keyspaceDefinition.Name;
            columnFamilyDefinition.Comparator_type = "BytesType";
            columnFamilyDefinition.Key_cache_size = 20000;
            columnFamilyDefinition.Row_cache_save_period_in_seconds = 0;
            columnFamilyDefinition.Key_cache_save_period_in_seconds = 3600;
            columnFamilyDefinition.Memtable_flush_after_mins = 59;
            columnFamilyDefinition.Memtable_throughput_in_mb = 255;
            columnFamilyDefinition.Memtable_operations_in_millions = 0.29;
            columnFamilyDefinition.Name = COLUMNFAMILYNAME;

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            ExecutionBlock describe_keyspaces = new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.describe_keyspaces();
            });

            List<KsDef> keyspaceDefinitions = (List<KsDef>)cluster.Execute(describe_keyspaces);
            bool keyspaceExists = false;
            bool columnFamilyExists = false;
            if (keyspaceDefinitions != null)
            {
                foreach (KsDef ksDef in keyspaceDefinitions)
                {
                    if (ksDef.Name.CompareTo(keyspaceDefinition.Name) == 0)
                    {
                        keyspaceExists = true;
                        List<CfDef> columnFamilyDefinitions = ksDef.Cf_defs;
                        if (columnFamilyDefinitions != null)
                        {
                            foreach (CfDef cfDef in columnFamilyDefinitions)
                            {
                                if (cfDef.Name.CompareTo(columnFamilyDefinition.Name) == 0)
                                {
                                    columnFamilyExists = true;
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }
            }
            if (!keyspaceExists)
            {
                ExecutionBlock create_keyspace = new ExecutionBlock(delegate(CassandraClient client)
                {
                    return client.system_add_keyspace(keyspaceDefinition);
                });
                cluster.Execute(create_keyspace);
            }

            if (!columnFamilyExists)
            {
                ExecutionBlock create_columnFamily = new ExecutionBlock(delegate(CassandraClient client)
                {
                    return client.system_add_column_family(columnFamilyDefinition);
                });
                cluster.Execute(create_columnFamily, keyspaceDefinition.Name);
            }
        }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        public static void MyClassCleanup()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            ExecutionBlock drop_keyspace = new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.system_drop_keyspace(KEYSPACENAME);
            });
            cluster.Execute(drop_keyspace);
        }

        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize()
        {
            PopulateDataWithInserts();
            PopulateDataWithBatchMutate();
        }

        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        private void PopulateDataWithInserts()
        {
            for (long i = 0; i < 100; i++)
            {
                byte[] key = ByteEncoderHelper.LongEncoder.ToByteArray(i);
                ColumnParent columnParent = new ColumnParent();
                Column column = new Column() {
                    Name = ByteEncoderHelper.UTF8Encoder.ToByteArray("Data"),
                    Timestamp = UnixHelper.UnixTimestamp,
                    Value = key
                };

                columnParent.Column_family = COLUMNFAMILYNAME;

                ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
                cluster.Execute(new ExecutionBlock(delegate(CassandraClient client) {
                    client.insert(key, columnParent, column, ConsistencyLevel.ONE);
                    return null;
                }), KEYSPACENAME);
            }
        }

        private void PopulateDataWithBatchMutate()
        {
            Dictionary<byte[], Dictionary<string,List<Mutation>>> mutation_map = new Dictionary<byte[],Dictionary<string,List<Mutation>>>();
            for (long i = 0; i < 100; i++)
            {
                byte[] key = ByteEncoderHelper.LongEncoder.ToByteArray(i);
                Dictionary<string,List<Mutation>> cfMutation = new Dictionary<string,List<Mutation>>();
                List<Mutation> mutationList = new List<Mutation>();
                for (long j = 0; j < 100; j++)
                {
                    string columnName = String.Format("Data-{0:0000000000}", j);
                    Mutation mutation = new Mutation()
                    {
                        Column_or_supercolumn = new ColumnOrSuperColumn()
                        {
                            Column = new Column()
                            {
                                Name = ByteEncoderHelper.UTF8Encoder.ToByteArray(columnName),
                                Timestamp = UnixHelper.UnixTimestamp,
                                Value = ByteEncoderHelper.LongEncoder.ToByteArray(j),
                            },
                        },
                    };
                    mutationList.Add(mutation);
                }
                cfMutation.Add(COLUMNFAMILYNAME, mutationList);
                mutation_map.Add(key, cfMutation);
            }

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client) {
                client.batch_mutate(mutation_map, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);
        }

        [TestMethod]
        public void testBatchMutateInsert()
        {
            byte[] key = ByteEncoderHelper.LongEncoder.ToByteArray(1);
            ColumnParent columnParent = new ColumnParent()
            {
                Column_family = COLUMNFAMILYNAME,
            };

            SlicePredicate predicate = new SlicePredicate()
            {
                Slice_range = new SliceRange()
                {
                    Count = 1000,
                    Reversed = false,
                    Start = new byte[0],
                    Finish = new byte[0],
                },
            };
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            object rtnValue = cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.get_count(key, columnParent, predicate, ConsistencyLevel.ONE);
            }), KEYSPACENAME);

            Int32 intValue = (Int32)rtnValue;
            Assert.AreEqual(101, intValue);
        }

        [TestMethod]
        public void TestBatchMutateOneColumn()
        {
            byte[] key = ByteEncoderHelper.LongEncoder.ToByteArray(2);
            ColumnPath columnPath = new ColumnPath()
            {
                Column = ByteEncoderHelper.UTF8Encoder.ToByteArray("Data-0000000000"),
                Column_family = COLUMNFAMILYNAME,
            };
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            object rtnValue = cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                return client.get(key, columnPath, ConsistencyLevel.ONE);
            }), KEYSPACENAME);


            Assert.IsNotNull(rtnValue);
        }

        //[TestMethod]
        public void SimplestTestWithKeyRange()
        {
            ColumnParent columnParent = new ColumnParent()
            {
                Column_family = COLUMNFAMILYNAME
            };

            SlicePredicate predicate = new SlicePredicate()
            {
                Slice_range = new SliceRange()
                {
                    Reversed = false,
                    Count = int.MaxValue,
                    Start = new byte[0]
                },
            };

            KeyRange keyRange = new KeyRange()
            {
                Count = 10,
                Start_key = ByteEncoderHelper.LongEncoder.ToByteArray(0),
                End_key = ByteEncoderHelper.LongEncoder.ToByteArray(50)
            };

            List<KeySlice> output = null;
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client) {
                output = client.get_range_slices(columnParent, predicate, keyRange, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            Assert.IsNotNull(output);
        }

        [TestMethod]
        public void TestWithKeyRangeGettingAllKeys()
        {
            ColumnParent columnParent = new ColumnParent()
            {
                Column_family = COLUMNFAMILYNAME
            };

            SlicePredicate predicate = new SlicePredicate()
            {
                Slice_range = new SliceRange()
                {
                    Reversed = false,
                    Count = int.MaxValue,
                    Start = new byte[0],
                    Finish = new byte[0]
                },
            };

            KeyRange keyRange = new KeyRange()
            {
                Count = 10,
                Start_token = "1",
                End_token = "0",
                
            };

            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);
            List<KeySlice> output = (List<KeySlice>) cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                output = client.get_range_slices(columnParent, predicate, keyRange, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            //Assert.IsNotNull(output);
        }

        [TestMethod]
        public void SimpleTestInsertCheckDeleteCheck()
        {
            ICluster cluster = AquilesHelper.RetrieveCluster(CLUSTERNAME);

            byte[] key = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            byte[] columnName = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            byte[] columnValue = ByteEncoderHelper.UTF8Encoder.ToByteArray(Guid.NewGuid().ToString());
            ColumnParent columnParent = new ColumnParent()
            {
                Column_family = COLUMNFAMILYNAME,
            };
            Column columnData = new Column()
            {
                Name = columnName,
                Timestamp = UnixHelper.UnixTimestamp,
                Value = columnValue,
            };

            // inserting data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                client.insert(key, columnParent, columnData, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            ColumnPath columnPath = new ColumnPath()
            {
                Column = columnName,
                Column_family = COLUMNFAMILYNAME,
            };
            ColumnOrSuperColumn columnOrSuperColumn = null;

            // getting inserted data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                columnOrSuperColumn = client.get(key, columnPath, ConsistencyLevel.ONE);
                return columnOrSuperColumn;
            }), KEYSPACENAME);

            Assert.IsNotNull(columnOrSuperColumn);
            Assert.IsNotNull(columnOrSuperColumn.Column);
            //Assert.AreEqual(columnOrSuperColumn.Column.Name, columnName);
            //Assert.AreEqual(columnOrSuperColumn.Column.Value, columnValue);

            // deleting data
            cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
            {
                client.remove(key, columnPath, UnixHelper.UnixTimestamp, ConsistencyLevel.ONE);
                return null;
            }), KEYSPACENAME);

            // verifying getting deleted data (there shouldn't be a response
            try
            {
                cluster.Execute(new ExecutionBlock(delegate(CassandraClient client)
                {
                    columnOrSuperColumn = client.get(key, columnPath, ConsistencyLevel.ONE);
                    return columnOrSuperColumn;
                }), KEYSPACENAME);
            }
            catch (ExecutionBlockException e)
            {
                Assert.AreEqual(e.InnerException.GetType(), typeof(NotFoundException));
                columnOrSuperColumn = null;
            }
            Assert.IsNull(columnOrSuperColumn);
        }
    }
}
